@@ -21,8 +21,8 @@
 #include <config.h>
 #endif
 
+#include <math.h>
 #include <gtk/gtk.h>
-#include <exo/exo.h>
 
 #include <common/panel-private.h>
 #include <libxfce4panel/libxfce4panel.h>
@@ -38,42 +38,49 @@ typedef struct _PanelItembarChild PanelItembarChild;
 
 
 
-static void               panel_itembar_set_property       (GObject         *object,
-                                                            guint            prop_id,
-                                                            const GValue    *value,
-                                                            GParamSpec      *pspec);
-static void               panel_itembar_get_property       (GObject         *object,
-                                                            guint            prop_id,
-                                                            GValue          *value,
-                                                            GParamSpec      *pspec);
-static void               panel_itembar_finalize           (GObject         *object);
-static void               panel_itembar_size_request       (GtkWidget       *widget,
-                                                            GtkRequisition  *requisition);
-static void               panel_itembar_size_allocate      (GtkWidget       *widget,
-                                                            GtkAllocation   *allocation);
-static gboolean           panel_itembar_expose_event       (GtkWidget       *widget,
-                                                            GdkEventExpose  *event);
-static void               panel_itembar_add                (GtkContainer    *container,
-                                                            GtkWidget       *child);
-static void               panel_itembar_remove             (GtkContainer    *container,
-                                                            GtkWidget       *child);
-static void               panel_itembar_forall             (GtkContainer    *container,
-                                                            gboolean         include_internals,
-                                                            GtkCallback      callback,
-                                                            gpointer         callback_data);
-static GType              panel_itembar_child_type         (GtkContainer    *container);
-static void               panel_itembar_set_child_property (GtkContainer    *container,
-                                                            GtkWidget       *widget,
-                                                            guint            prop_id,
-                                                            const GValue    *value,
-                                                            GParamSpec      *pspec);
-static void               panel_itembar_get_child_property (GtkContainer    *container,
-                                                            GtkWidget       *widget,
-                                                            guint            prop_id,
-                                                            GValue          *value,
-                                                            GParamSpec      *pspec);
-static PanelItembarChild *panel_itembar_get_child          (PanelItembar    *itembar,
-                                                            GtkWidget       *widget);
+static void               panel_itembar_set_property         (GObject         *object,
+                                                              guint            prop_id,
+                                                              const GValue    *value,
+                                                              GParamSpec      *pspec);
+static void               panel_itembar_get_property         (GObject         *object,
+                                                              guint            prop_id,
+                                                              GValue          *value,
+                                                              GParamSpec      *pspec);
+static void               panel_itembar_finalize             (GObject         *object);
+static void               panel_itembar_get_preferred_length (GtkWidget       *widget,
+                                                              gint            *minimum_length,
+                                                              gint            *natural_length);
+static void               panel_itembar_get_preferred_width  (GtkWidget       *widget,
+                                                              gint            *minimum_width,
+                                                              gint            *natural_width);
+static void               panel_itembar_get_preferred_height (GtkWidget       *widget,
+                                                              gint            *minimum_height,
+                                                              gint            *natural_height);
+static void               panel_itembar_size_allocate        (GtkWidget       *widget,
+                                                              GtkAllocation   *allocation);
+static gboolean           panel_itembar_draw                 (GtkWidget *widget,
+                                                              cairo_t   *cr);
+static void               panel_itembar_add                  (GtkContainer    *container,
+                                                              GtkWidget       *child);
+static void               panel_itembar_remove               (GtkContainer    *container,
+                                                              GtkWidget       *child);
+static void               panel_itembar_forall               (GtkContainer    *container,
+                                                              gboolean         include_internals,
+                                                              GtkCallback      callback,
+                                                              gpointer         callback_data);
+static GType              panel_itembar_child_type           (GtkContainer    *container);
+static void               panel_itembar_set_child_property   (GtkContainer    *container,
+                                                              GtkWidget       *widget,
+                                                              guint            prop_id,
+                                                              const GValue    *value,
+                                                              GParamSpec      *pspec);
+static void               panel_itembar_get_child_property   (GtkContainer    *container,
+                                                              GtkWidget       *widget,
+                                                              guint            prop_id,
+                                                              GValue          *value,
+                                                              GParamSpec      *pspec);
+static PanelItembarChild *panel_itembar_get_child            (PanelItembar    *itembar,
+                                                              GtkWidget       *widget);
 
 
 
@@ -95,7 +102,7 @@ struct _PanelItembar
 
   /* dnd support */
   gint                 highlight_index;
-  gint                 highlight_x, highlight_y;
+  gint                 highlight_x, highlight_y, highlight_length;
   gboolean             highlight_small;
 };
 
@@ -160,9 +167,10 @@ panel_itembar_class_init (PanelItembarClass *klass)
   gobject_class->finalize = panel_itembar_finalize;
 
   gtkwidget_class = GTK_WIDGET_CLASS (klass);
-  gtkwidget_class->size_request = panel_itembar_size_request;
+  gtkwidget_class->get_preferred_width = panel_itembar_get_preferred_width;
+  gtkwidget_class->get_preferred_height = panel_itembar_get_preferred_height;
   gtkwidget_class->size_allocate = panel_itembar_size_allocate;
-  gtkwidget_class->expose_event = panel_itembar_expose_event;
+  gtkwidget_class->draw = panel_itembar_draw;
 
   gtkcontainer_class = GTK_CONTAINER_CLASS (klass);
   gtkcontainer_class->add = panel_itembar_add;
@@ -186,42 +194,42 @@ panel_itembar_class_init (PanelItembarClass *klass)
                                                       NULL, NULL,
                                                       XFCE_TYPE_PANEL_PLUGIN_MODE,
                                                       XFCE_PANEL_PLUGIN_MODE_HORIZONTAL,
-                                                      EXO_PARAM_WRITABLE));
+                                                      G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class,
                                    PROP_SIZE,
                                    g_param_spec_uint ("size",
                                                       NULL, NULL,
                                                       16, 128, 30,
-                                                      EXO_PARAM_WRITABLE));
+                                                      G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class,
                                    PROP_NROWS,
                                    g_param_spec_uint ("nrows",
                                                       NULL, NULL,
                                                       1, 6, 1,
-                                                      EXO_PARAM_WRITABLE));
+                                                      G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS));
 
   gtk_container_class_install_child_property (gtkcontainer_class,
                                               CHILD_PROP_EXPAND,
                                               g_param_spec_boolean ("expand",
                                                                     NULL, NULL,
                                                                     FALSE,
-                                                                    EXO_PARAM_READWRITE));
+                                                                    G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gtk_container_class_install_child_property (gtkcontainer_class,
                                               CHILD_PROP_SHRINK,
                                               g_param_spec_boolean ("shrink",
                                                                     NULL, NULL,
                                                                     FALSE,
-                                                                    EXO_PARAM_READWRITE));
+                                                                    G_PARAM_READWRITE| G_PARAM_STATIC_STRINGS));
 
   gtk_container_class_install_child_property (gtkcontainer_class,
                                               CHILD_PROP_SMALL,
                                               g_param_spec_boolean ("small",
                                                                     NULL, NULL,
                                                                     FALSE,
-                                                                    EXO_PARAM_READWRITE));
+                                                                    G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 }
 
 
@@ -234,8 +242,9 @@ panel_itembar_init (PanelItembar *itembar)
   itembar->size = 30;
   itembar->nrows = 1;
   itembar->highlight_index = -1;
+  itembar->highlight_length = -1;
 
-  GTK_WIDGET_SET_FLAGS (GTK_WIDGET (itembar), GTK_NO_WINDOW);
+  gtk_widget_set_has_window (GTK_WIDGET (itembar), FALSE);
 
   gtk_widget_set_redraw_on_allocate (GTK_WIDGET (itembar), FALSE);
 }
@@ -298,29 +307,27 @@ panel_itembar_finalize (GObject *object)
   (*G_OBJECT_CLASS (panel_itembar_parent_class)->finalize) (object);
 }
 
-#define CHILD_LENGTH(child_req, itembar) \
-  (IS_HORIZONTAL (itembar) ? child_req.width : child_req.height)
-
 static void
-panel_itembar_size_request (GtkWidget      *widget,
-                            GtkRequisition *requisition)
+panel_itembar_get_preferred_length (GtkWidget      *widget,
+                                    gint           *minimum_length,
+                                    gint           *natural_length)
 {
   PanelItembar      *itembar = PANEL_ITEMBAR (widget);
   GSList            *li;
   PanelItembarChild *child;
-  GtkRequisition     child_req;
   gint               border_width;
-  gint               row_max_size;
+  gint               row_max_size, row_max_size_min;
   gint               col_count;
-  gint               rows_size;
-  gint               total_len;
-  gint               child_len;
+  gint               total_len, total_len_min;
+  gint               child_len, child_len_min;
 
   /* total length we request */
   total_len = 0;
+  total_len_min = 0;
 
   /* counter for small child packing */
   row_max_size = 0;
+  row_max_size_min = 0;
   col_count = 0;
 
   for (li = itembar->children; li != NULL; li = li->next)
@@ -329,24 +336,26 @@ panel_itembar_size_request (GtkWidget      *widget,
 
       if (G_LIKELY (child != NULL))
         {
-          if (!GTK_WIDGET_VISIBLE (child->widget))
+          if (!gtk_widget_get_visible (child->widget))
             continue;
 
           /* get the child's size request */
-          gtk_widget_size_request (child->widget, &child_req);
-
+          if (IS_HORIZONTAL (itembar))
+            gtk_widget_get_preferred_width (child->widget, &child_len_min, &child_len);
+          else
+            gtk_widget_get_preferred_height (child->widget, &child_len_min, &child_len);
           /* check if the small child fits in a row */
           if (child->option == CHILD_OPTION_SMALL
               && itembar->nrows > 1)
             {
-              child_len = CHILD_LENGTH (child_req, itembar);
-
               /* make sure we have enough space for all the children on the row.
                * so add the difference between the largest child in this column */
               if (child_len > row_max_size)
                 {
                   total_len += child_len - row_max_size;
+                  total_len_min += child_len_min - row_max_size_min;
                   row_max_size = child_len;
+                  row_max_size_min = child_len_min;
                 }
 
               /* reset to new row if all columns are filled */
@@ -354,38 +363,91 @@ panel_itembar_size_request (GtkWidget      *widget,
                 {
                   col_count = 0;
                   row_max_size = 0;
+                  row_max_size_min = 0;
                 }
             }
           else /* expanding or normal item */
             {
-              total_len += CHILD_LENGTH (child_req, itembar);
+              total_len += child_len;
+              total_len_min += child_len_min;
 
               /* reset column packing */
               col_count = 0;
               row_max_size = 0;
+              row_max_size_min = 0;
             }
         }
       else
         {
           /* this noop item is the dnd position */
           total_len += HIGHLIGHT_SIZE;
+          total_len_min += HIGHLIGHT_SIZE;
         }
     }
 
-  /* the size property stored in the itembar is that of a single row */
-  rows_size = itembar->size * itembar->nrows;
-
   /* return the total size */
-  border_width = GTK_CONTAINER (widget)->border_width * 2;
+  border_width = gtk_container_get_border_width (GTK_CONTAINER (widget)) * 2;
+  total_len += border_width;
+  total_len_min += border_width;
+
+  if (natural_length != NULL)
+    *natural_length = total_len;
+
+  if (minimum_length != NULL)
+    *minimum_length = total_len_min;
+}
+
+
+
+static void
+panel_itembar_get_preferred_width (GtkWidget *widget,
+                                   gint      *minimum_width,
+                                   gint      *natural_width)
+{
+  PanelItembar   *itembar = PANEL_ITEMBAR (widget);
+  gint            size;
+
   if (IS_HORIZONTAL (itembar))
     {
-      requisition->width = total_len + border_width;
-      requisition->height = rows_size + border_width;
+      panel_itembar_get_preferred_length (widget, minimum_width, natural_width);
     }
   else
     {
-      requisition->height = total_len + border_width;
-      requisition->width = rows_size + border_width;
+      size = itembar->size * itembar->nrows +
+        gtk_container_get_border_width (GTK_CONTAINER (widget)) * 2;
+
+      if (minimum_width != NULL)
+        *minimum_width = size;
+
+      if (natural_width != NULL)
+        *natural_width = size;
+    }
+}
+
+
+
+static void
+panel_itembar_get_preferred_height (GtkWidget *widget,
+                                    gint      *minimum_height,
+                                    gint      *natural_height)
+{
+  PanelItembar   *itembar = PANEL_ITEMBAR (widget);
+  gint            size;
+
+  if (IS_HORIZONTAL (itembar))
+    {
+      size = itembar->size * itembar->nrows +
+        gtk_container_get_border_width (GTK_CONTAINER (widget)) * 2;
+
+      if (minimum_height != NULL)
+        *minimum_height = size;
+
+      if (natural_height != NULL)
+        *natural_height = size;
+    }
+  else
+    {
+      panel_itembar_get_preferred_length (widget, minimum_height, natural_height);
     }
 }
 
@@ -398,7 +460,6 @@ panel_itembar_size_allocate (GtkWidget     *widget,
   PanelItembar      *itembar = PANEL_ITEMBAR (widget);
   GSList            *lp, *ltemp;
   PanelItembarChild *child;
-  GtkRequisition     child_req;
   GtkAllocation      child_alloc;
   gint               border_width;
   gint               expand_len_avail, expand_len_req;
@@ -407,17 +468,21 @@ panel_itembar_size_allocate (GtkWidget     *widget,
   gint               x, y;
   gint               x_init, y_init;
   gboolean           expand_children_fit;
-  gint               new_len;
-  gint               child_len;
+  gint               new_len, sub_len;
+  gint               child_len, child_len_min;
   gint               row_max_size;
   gint               col_count;
   gint               rows_size;
 
+  #define CHILD_MIN_ALLOC_LEN(child_len) \
+    if (G_UNLIKELY ((child_len) < 1)) \
+      (child_len) = 1;
+
   /* the maximum allocation is limited by that of the
    * panel window, so take over the assigned allocation */
-  widget->allocation = *allocation;
+  gtk_widget_set_allocation (widget, allocation);
 
-  border_width = GTK_CONTAINER (widget)->border_width;
+  border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
 
   if (IS_HORIZONTAL (itembar))
     itembar_len = allocation->width - 2 * border_width;
@@ -442,12 +507,17 @@ panel_itembar_size_allocate (GtkWidget     *widget,
       child = lp->data;
       if (G_LIKELY (child != NULL))
         {
-          if (!GTK_WIDGET_VISIBLE (child->widget))
+          if (!gtk_widget_get_visible (child->widget))
             continue;
 
-          gtk_widget_get_child_requisition (child->widget, &child_req);
+          if (IS_HORIZONTAL (itembar))
+            gtk_widget_get_preferred_width (child->widget, &child_len_min, &child_len);
+          else
+            gtk_widget_get_preferred_height (child->widget, &child_len_min, &child_len);
 
-          child_len = CHILD_LENGTH (child_req, itembar);
+          /* child will allocate at least 1 pixel */
+          CHILD_MIN_ALLOC_LEN (child_len);
+          CHILD_MIN_ALLOC_LEN (child_len_min);
 
           if (G_UNLIKELY (child->option == CHILD_OPTION_SMALL
                           && itembar->nrows > 1))
@@ -480,8 +550,8 @@ panel_itembar_size_allocate (GtkWidget     *widget,
                 {
                   expand_len_avail -= child_len;
 
-                  if (child->option == CHILD_OPTION_SHRINK)
-                    shrink_len_avail += child_len;
+                  if (child_len_min < child_len)
+                    shrink_len_avail += (child_len - child_len_min);
                 }
             }
         }
@@ -532,8 +602,8 @@ panel_itembar_size_allocate (GtkWidget     *widget,
 
           if (itembar->highlight_small)
             {
-              itembar->highlight_x = x;
-              itembar->highlight_y = y;
+              itembar->highlight_x = x - x_init;
+              itembar->highlight_y = y - y_init;
               if (IS_HORIZONTAL (itembar))
                 y += HIGHLIGHT_SIZE;
               else
@@ -541,16 +611,16 @@ panel_itembar_size_allocate (GtkWidget     *widget,
             }
           else if (IS_HORIZONTAL (itembar))
             {
-              itembar->highlight_x = (col_count > 0) ? x + row_max_size : x;
-              itembar->highlight_y = y_init;
+              itembar->highlight_x = ((col_count > 0) ? x + row_max_size : x) - x_init;
+              itembar->highlight_y = 0;
 
               x += HIGHLIGHT_SIZE;
               expand_len_avail -= HIGHLIGHT_SIZE;
             }
           else
             {
-              itembar->highlight_x = x_init;
-              itembar->highlight_y = (col_count > 0) ? y + row_max_size : y;
+              itembar->highlight_x = 0;
+              itembar->highlight_y = ((col_count > 0) ? y + row_max_size : y) - y_init;
 
               y += HIGHLIGHT_SIZE;
               expand_len_avail -= HIGHLIGHT_SIZE;
@@ -559,12 +629,13 @@ panel_itembar_size_allocate (GtkWidget     *widget,
           continue;
         }
 
-      if (!GTK_WIDGET_VISIBLE (child->widget))
+      if (!gtk_widget_get_visible (child->widget))
         continue;
 
-      gtk_widget_get_child_requisition (child->widget, &child_req);
-
-      child_len = CHILD_LENGTH (child_req, itembar);
+      if (IS_HORIZONTAL (itembar))
+        gtk_widget_get_preferred_width (child->widget, &child_len_min, &child_len);
+      else
+        gtk_widget_get_preferred_height (child->widget, &child_len_min, &child_len);
 
       if (G_UNLIKELY (!expand_children_fit && child->option == CHILD_OPTION_EXPAND))
         {
@@ -572,26 +643,38 @@ panel_itembar_size_allocate (GtkWidget     *widget,
           panel_assert (expand_len_req > 0);
           new_len = expand_len_avail * child_len / expand_len_req;
 
+          CHILD_MIN_ALLOC_LEN (child_len);
+          CHILD_MIN_ALLOC_LEN (child_len_min);
+          CHILD_MIN_ALLOC_LEN (new_len);
+
           expand_len_req -= child_len;
           expand_len_avail -= new_len;
 
           child_len = new_len;
         }
-      else if (child->option == CHILD_OPTION_SHRINK
+      else if (child_len_min < child_len
                && shrink_len_req > 0)
         {
           /* equally shrink all shrinking plugins */
           panel_assert (shrink_len_avail > 0);
-          new_len = shrink_len_req * child_len / shrink_len_avail;
+          sub_len = MIN (shrink_len_req * (child_len - child_len_min) / shrink_len_avail,
+                         child_len - child_len_min);
+          new_len = child_len - sub_len;
 
-          shrink_len_req -= new_len;
-          shrink_len_avail -= child_len;
+          CHILD_MIN_ALLOC_LEN (child_len);
+          CHILD_MIN_ALLOC_LEN (child_len_min);
+          CHILD_MIN_ALLOC_LEN (new_len);
 
-          child_len -= new_len;
+          shrink_len_req -= sub_len;
+          shrink_len_avail -= (child_len - child_len_min);
+
+          child_len = new_len;
+        }
+      else
+        {
+          CHILD_MIN_ALLOC_LEN (child_len);
+          CHILD_MIN_ALLOC_LEN (child_len_min);
         }
-
-      if (G_UNLIKELY (child_len < 1))
-        child_len = 1;
 
       if (child->option == CHILD_OPTION_SMALL
           && itembar->nrows > 1)
@@ -679,17 +762,15 @@ panel_itembar_size_allocate (GtkWidget     *widget,
 
 
 static gboolean
-panel_itembar_expose_event (GtkWidget      *widget,
-                            GdkEventExpose *event)
+panel_itembar_draw (GtkWidget *widget,
+                    cairo_t   *cr)
 {
   PanelItembar *itembar = PANEL_ITEMBAR (widget);
   gboolean      result;
-  cairo_t      *cr;
-  GdkWindow    *window;
   GdkRectangle  rect;
   gint          row_size;
 
-  result = (*GTK_WIDGET_CLASS (panel_itembar_parent_class)->expose_event) (widget, event);
+  result = (*GTK_WIDGET_CLASS (panel_itembar_parent_class)->draw) (widget, cr);
 
   if (itembar->highlight_index != -1)
     {
@@ -702,26 +783,21 @@ panel_itembar_expose_event (GtkWidget      *widget,
           (!IS_HORIZONTAL (itembar) && itembar->highlight_small))
         {
           rect.width = HIGHLIGHT_SIZE;
-          rect.height = row_size;
+          rect.height = (itembar->highlight_length != -1) ?
+            itembar->highlight_length : row_size;
         }
       else
         {
           rect.height = HIGHLIGHT_SIZE;
-          rect.width = row_size;
+          rect.width = (itembar->highlight_length != -1) ?
+            itembar->highlight_length : row_size;
         }
 
       /* draw highlight box */
-      window = gtk_widget_get_window (widget);
-      cr = gdk_cairo_create (window);
       cairo_set_source_rgb (cr, 1.00, 0.00, 0.00);
 
-      gdk_cairo_rectangle (cr, &event->area);
-      cairo_clip (cr);
-
       gdk_cairo_rectangle (cr, &rect);
       cairo_fill (cr);
-
-      cairo_destroy (cr);
     }
 
   return result;
@@ -747,7 +823,7 @@ panel_itembar_remove (GtkContainer *container,
 
   panel_return_if_fail (PANEL_IS_ITEMBAR (itembar));
   panel_return_if_fail (GTK_IS_WIDGET (widget));
-  panel_return_if_fail (widget->parent == GTK_WIDGET (container));
+  panel_return_if_fail (gtk_widget_get_parent (widget) == GTK_WIDGET (container));
   panel_return_if_fail (itembar->children != NULL);
 
   child = panel_itembar_get_child (itembar, widget);
@@ -895,7 +971,7 @@ panel_itembar_get_child (PanelItembar *itembar,
 
   panel_return_val_if_fail (PANEL_IS_ITEMBAR (itembar), NULL);
   panel_return_val_if_fail (GTK_IS_WIDGET (widget), NULL);
-  panel_return_val_if_fail (widget->parent == GTK_WIDGET (itembar), NULL);
+  panel_return_val_if_fail (gtk_widget_get_parent (widget) == GTK_WIDGET (itembar), NULL);
 
   for (li = itembar->children; li != NULL; li = g_slist_next (li))
     {
@@ -927,7 +1003,7 @@ panel_itembar_insert (PanelItembar *itembar,
 
   panel_return_if_fail (PANEL_IS_ITEMBAR (itembar));
   panel_return_if_fail (GTK_IS_WIDGET (widget));
-  panel_return_if_fail (widget->parent == NULL);
+  panel_return_if_fail (gtk_widget_get_parent (widget) == NULL);
 
   child = g_slice_new0 (PanelItembarChild);
   child->widget = widget;
@@ -951,7 +1027,7 @@ panel_itembar_reorder_child (PanelItembar *itembar,
 
   panel_return_if_fail (PANEL_IS_ITEMBAR (itembar));
   panel_return_if_fail (GTK_IS_WIDGET (widget));
-  panel_return_if_fail (widget->parent == GTK_WIDGET (itembar));
+  panel_return_if_fail (gtk_widget_get_parent (widget) == GTK_WIDGET (itembar));
 
   child = panel_itembar_get_child (itembar, widget);
   if (G_LIKELY (child != NULL))
@@ -977,7 +1053,7 @@ panel_itembar_get_child_index (PanelItembar *itembar,
 
   panel_return_val_if_fail (PANEL_IS_ITEMBAR (itembar), -1);
   panel_return_val_if_fail (GTK_IS_WIDGET (widget), -1);
-  panel_return_val_if_fail (widget->parent == GTK_WIDGET (itembar), -1);
+  panel_return_val_if_fail (gtk_widget_get_parent (widget) == GTK_WIDGET (itembar), -1);
 
   for (idx = 0, li = itembar->children; li != NULL; li = g_slist_next (li), idx++)
     {
@@ -1015,16 +1091,17 @@ panel_itembar_get_drop_index (PanelItembar *itembar,
                               gint          x,
                               gint          y)
 {
-  PanelItembarChild *child;
-  GSList            *li;
+  PanelItembarChild *child, *child2;
+  GSList            *li, *li2;
   GtkAllocation      alloc;
-  guint              idx, col_start_idx;
-  gint               xr, yr;
+  guint              idx, col_start_idx, col_end_idx;
+  gint               xr, yr, col_width;
+  gdouble            aspect;
 
   panel_return_val_if_fail (PANEL_IS_ITEMBAR (itembar), 0);
 
   /* add the itembar position */
-  alloc = GTK_WIDGET (itembar)->allocation;
+  gtk_widget_get_allocation (GTK_WIDGET (itembar), &alloc);
 
   if (!IS_HORIZONTAL (itembar))
     {
@@ -1037,47 +1114,81 @@ panel_itembar_get_drop_index (PanelItembar *itembar,
       x >= alloc.x + alloc.width || y >= alloc.y + alloc.height)
     return g_slist_length (itembar->children);
 
-  for (li = itembar->children, idx = 0, col_start_idx = 0; li != NULL; li = g_slist_next (li))
+  col_width = -1;
+  itembar->highlight_length = -1;
+  idx = 0;
+  col_start_idx = 0;
+  col_end_idx = 0;
+
+  for (li = itembar->children; li != NULL; li = g_slist_next (li))
     {
       child = li->data;
       if (G_UNLIKELY (child == NULL))
         continue;
 
-      if (child->row == 0)
-        col_start_idx = idx;
-
-      alloc = child->widget->allocation;
+      panel_assert (child->widget != NULL);
+      gtk_widget_get_allocation (child->widget, &alloc);
 
       if (!IS_HORIZONTAL (itembar))
         TRANSPOSE_AREA (alloc);
 
       xr = x - alloc.x;
       yr = y - alloc.y;
+
       if (child->option == CHILD_OPTION_SMALL)
         {
+          /* are we at the beginning of the column? */
+          if (child->row == 0)
+            {
+              col_start_idx = idx;
+              col_end_idx = idx + 1;
+              col_width = alloc.width;
+              /* find the width of the current column and the idx of last item */
+              for (li2 = g_slist_next (li); li2 != NULL; li2 = g_slist_next (li2))
+                {
+                  child2 = li2->data;
+                  if (G_UNLIKELY (child2 == NULL))
+                    continue;
+                  if (child2->row == 0)
+                    break;
+                  panel_assert (child2->widget != NULL);
+                  col_end_idx++;
+                  if (IS_HORIZONTAL (itembar))
+                    col_width = MAX (col_width, gtk_widget_get_allocated_width (child2->widget));
+                  else
+                    col_width = MAX (col_width, gtk_widget_get_allocated_height (child2->widget));
+                }
+            }
+
+          /* calculate aspect ratio */
+          if (alloc.height > 0 && col_width > 0)
+            aspect = (gdouble) col_width / (gdouble) alloc.height;
+          else
+            aspect = 1.0;
+
           /* before current column */
           if (xr < 0 ||
-              (xr < yr && xr < alloc.height - yr))
+              (xr < (gint) round (yr * aspect) &&
+               xr < (gint) round ((alloc.height - yr) * aspect)))
             {
               idx = col_start_idx;
               break;
             }
           /* before current child */
-          if (xr < alloc.width && xr >= yr && alloc.width - xr >= yr)
-            break;
-          /* after current column */
-          if (xr < alloc.width && xr >= alloc.height - yr && xr >= yr)
+          if (xr < col_width &&
+              xr >= (gint) round (yr * aspect) &&
+              col_width - xr >= (gint) round (yr * aspect))
             {
-              /* search for the beginning of the next column */
-              for (li = g_slist_next (li), idx++; li != NULL; li = g_slist_next (li))
-                {
-                  child = li->data;
-                  if (G_UNLIKELY (child == NULL))
-                    continue;
-                  if (child->row == 0)
-                    break;
-                  idx++;
-                }
+              if (child->row != 0)
+                itembar->highlight_length = col_width;
+              break;
+            }
+          /* after current column */
+          if (xr < col_width &&
+              xr >= (gint) round ((alloc.height - yr) * aspect) &&
+              xr >= (gint) round (yr * aspect))
+            {
+              idx = col_end_idx;
               break;
             }
         }
