@@ -113,9 +113,10 @@ typedef enum
   ACTION_TYPE_SWITCH_USER   = 1 << 4,
   ACTION_TYPE_LOCK_SCREEN   = 1 << 5,
   ACTION_TYPE_HIBERNATE     = 1 << 6,
-  ACTION_TYPE_SUSPEND       = 1 << 7,
-  ACTION_TYPE_RESTART       = 1 << 8,
-  ACTION_TYPE_SHUTDOWN      = 1 << 9
+  ACTION_TYPE_HYBRID_SLEEP  = 1 << 7,
+  ACTION_TYPE_SUSPEND       = 1 << 8,
+  ACTION_TYPE_RESTART       = 1 << 9,
+  ACTION_TYPE_SHUTDOWN      = 1 << 10
 }
 ActionType;
 
@@ -183,6 +184,14 @@ static ActionEntry action_entries[] =
     N_("Hibernating computer in %d seconds."),
     "system-hibernate"
   },
+  { ACTION_TYPE_HYBRID_SLEEP,
+    "hybrid-sleep",
+    N_("Hybrid Sleep"),
+    N_("_Hybrid Sleep"),
+    N_("Do you want to hibernate and suspend the system?"),
+    N_("Hibernating and Suspending computer in %d seconds."),
+    "system-hibernate"
+  },
   { ACTION_TYPE_SUSPEND,
     "suspend",
     N_("Suspend"),
@@ -282,6 +291,17 @@ actions_plugin_init (ActionsPlugin *plugin)
 
 
 
+static void
+actions_plugin_free_array_element (gpointer data)
+{
+  GValue *value = (GValue *)data;
+
+  g_value_unset (value);
+  g_free (value);
+}
+
+
+
 static void
 actions_plugin_get_property (GObject    *object,
                              guint       prop_id,
@@ -327,8 +347,9 @@ actions_plugin_set_property (GObject      *object,
   switch (prop_id)
     {
     case PROP_ITEMS:
-      if (plugin->items != NULL)
-        xfconf_array_free (plugin->items);
+      if (plugin->items != NULL) {
+        g_ptr_array_unref (plugin->items);
+      }
       plugin->items = g_value_dup_boxed (value);
       actions_plugin_pack (plugin);
       break;
@@ -394,7 +415,7 @@ actions_plugin_free_data (XfcePanelPlugin *panel_plugin)
     g_source_remove (plugin->pack_idle_id);
 
   if (plugin->items != NULL)
-    xfconf_array_free (plugin->items);
+    g_ptr_array_unref (plugin->items);
 
   if (plugin->menu != NULL)
     gtk_widget_destroy (plugin->menu);
@@ -454,7 +475,7 @@ actions_plugin_configure_store (gpointer data)
   model = g_object_get_data (G_OBJECT (plugin), "items-store");
   panel_return_val_if_fail (GTK_IS_LIST_STORE (model), FALSE);
 
-  array = g_ptr_array_new ();
+  array = g_ptr_array_new_full (1, (GDestroyNotify) actions_plugin_free_array_element);
 
   if (gtk_tree_model_get_iter_first (model, &iter))
     {
@@ -479,7 +500,7 @@ actions_plugin_configure_store (gpointer data)
 
   /* Store the new array */
   if (plugin->items != NULL)
-    xfconf_array_free (plugin->items);
+    g_ptr_array_unref (plugin->items);
   plugin->items = array;
   g_object_notify (G_OBJECT (plugin), "items");
 
@@ -794,7 +815,8 @@ actions_plugin_action_dbus_xfsm (const gchar  *method,
                                            error);
         }
       else if (g_strcmp0 (method, "Suspend") == 0
-               || g_strcmp0 (method, "Hibernate") == 0)
+               || g_strcmp0 (method, "Hibernate") == 0
+               || g_strcmp0 (method, "HybridSleep") == 0)
         {
           retval = g_dbus_proxy_call_sync (proxy, method,
                                            NULL,
@@ -902,6 +924,9 @@ actions_plugin_actions_allowed (void)
           if (actions_plugin_action_dbus_can (proxy, "CanHibernate"))
             PANEL_SET_FLAG (allow_mask, ACTION_TYPE_HIBERNATE);
 
+          if (actions_plugin_action_dbus_can (proxy, "CanHybridSleep"))
+            PANEL_SET_FLAG (allow_mask, ACTION_TYPE_HYBRID_SLEEP);
+
           g_object_unref (G_OBJECT (proxy));
         }
     }
@@ -968,6 +993,11 @@ actions_plugin_action_activate (GtkWidget      *widget,
                                                  FALSE, &error);
       break;
 
+    case ACTION_TYPE_HYBRID_SLEEP:
+      succeed = actions_plugin_action_dbus_xfsm ("HybridSleep", FALSE,
+                                                 FALSE, &error);
+      break;
+
     case ACTION_TYPE_SUSPEND:
       succeed = actions_plugin_action_dbus_xfsm ("Suspend", FALSE,
                                                  FALSE, &error);
@@ -1217,6 +1247,7 @@ actions_plugin_default_array (void)
       "+separator",
       "+suspend",
       "-hibernate",
+      "-hybrid-sleep",
       "-separator",
       "+shutdown",
       "-restart",
