@@ -33,7 +33,6 @@
 #include <common/panel-utils.h>
 #include <common/panel-private.h>
 #include <libwnck/libwnck.h>
-#include <exo/exo.h>
 
 #include "pager.h"
 #include "pager-buttons.h"
@@ -53,13 +52,13 @@ static void     pager_plugin_set_property                 (GObject           *ob
                                                            guint              prop_id,
                                                            const GValue      *value,
                                                            GParamSpec        *pspec);
-static void     pager_plugin_size_request                 (GtkWidget         *widget,
-                                                           GtkRequisition    *requisition);
 static gboolean pager_plugin_scroll_event                 (GtkWidget         *widget,
                                                            GdkEventScroll    *event);
 static void     pager_plugin_screen_changed               (GtkWidget         *widget,
                                                            GdkScreen         *previous_screen);
 static void     pager_plugin_construct                    (XfcePanelPlugin   *panel_plugin);
+static void     pager_plugin_style_updated                (GtkWidget         *pager,
+                                                           gpointer           user_data);
 static void     pager_plugin_free_data                    (XfcePanelPlugin   *panel_plugin);
 static gboolean pager_plugin_size_changed                 (XfcePanelPlugin   *panel_plugin,
                                                            gint               size);
@@ -68,6 +67,20 @@ static void     pager_plugin_mode_changed                 (XfcePanelPlugin     *
 static void     pager_plugin_configure_workspace_settings (GtkWidget         *button);
 static void     pager_plugin_configure_plugin             (XfcePanelPlugin   *panel_plugin);
 static void     pager_plugin_screen_layout_changed        (PagerPlugin       *plugin);
+static void     pager_plugin_get_preferred_width          (GtkWidget           *widget,
+                                                           gint                *minimum_width,
+                                                           gint                *natural_width);
+static void     pager_plugin_get_preferred_height         (GtkWidget           *widget,
+                                                           gint                *minimum_height,
+                                                           gint                *natural_height);
+static void     pager_plugin_get_preferred_width_for_height (GtkWidget           *widget,
+                                                             gint                 height,
+                                                             gint                *minimum_width,
+                                                             gint                *natural_width);
+static void     pager_plugin_get_preferred_height_for_width (GtkWidget           *widget,
+                                                             gint                 width,
+                                                             gint                *minimum_height,
+                                                             gint                *natural_height);
 
 
 
@@ -122,7 +135,10 @@ pager_plugin_class_init (PagerPluginClass *klass)
 
   widget_class = GTK_WIDGET_CLASS (klass);
   widget_class->scroll_event = pager_plugin_scroll_event;
-  widget_class->size_request = pager_plugin_size_request;
+  widget_class->get_preferred_width = pager_plugin_get_preferred_width;
+  widget_class->get_preferred_width_for_height  = pager_plugin_get_preferred_width_for_height;
+  widget_class->get_preferred_height = pager_plugin_get_preferred_height;
+  widget_class->get_preferred_height_for_width  = pager_plugin_get_preferred_height_for_width;
 
   plugin_class = XFCE_PANEL_PLUGIN_CLASS (klass);
   plugin_class->construct = pager_plugin_construct;
@@ -136,28 +152,28 @@ pager_plugin_class_init (PagerPluginClass *klass)
                                    g_param_spec_boolean ("workspace-scrolling",
                                                          NULL, NULL,
                                                          TRUE,
-                                                         EXO_PARAM_READWRITE));
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class,
                                    PROP_WRAP_WORKSPACES,
                                    g_param_spec_boolean ("wrap-workspaces",
                                                          NULL, NULL,
                                                          FALSE,
-                                                         EXO_PARAM_READWRITE));
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class,
                                    PROP_MINIATURE_VIEW,
                                    g_param_spec_boolean ("miniature-view",
                                                          NULL, NULL,
                                                          TRUE,
-                                                         EXO_PARAM_READWRITE));
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class,
                                    PROP_ROWS,
                                    g_param_spec_uint ("rows",
                                                       NULL, NULL,
                                                       1, 50, 1,
-                                                      EXO_PARAM_READWRITE));
+                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 }
 
 
@@ -258,6 +274,42 @@ pager_plugin_set_property (GObject      *object,
 
 
 
+static void
+pager_plugin_style_updated (GtkWidget *pager,
+                            gpointer   user_data)
+{
+  GtkWidget               *toplevel = gtk_widget_get_toplevel (pager);
+  GtkStyleContext         *context;
+  GtkCssProvider          *provider;
+  GdkRGBA                 *bg_color;
+  gchar                   *css_string;
+  gchar                   *color_string;
+
+  g_return_if_fail (gtk_widget_is_toplevel (toplevel));
+
+  /* Get the background color of the panel to draw selected and hover states */
+  provider = gtk_css_provider_new ();
+  context = gtk_widget_get_style_context (GTK_WIDGET (toplevel));
+  gtk_style_context_get (context, GTK_STATE_FLAG_NORMAL,
+                         GTK_STYLE_PROPERTY_BACKGROUND_COLOR,
+                         &bg_color, NULL);
+  color_string = gdk_rgba_to_string(bg_color);
+  // FIXME: The shade value only works well visually for bright themes/panels
+  css_string = g_strdup_printf ("wnck-pager { background: %s; }"
+                                "wnck-pager:selected { background: shade(%s, 0.7); }"
+                                "wnck-pager:hover { background: shade(%s, 0.9); }",
+                                color_string, color_string, color_string);
+  context = gtk_widget_get_style_context (pager);
+  gtk_css_provider_load_from_data (provider, css_string, -1, NULL);
+  gtk_style_context_add_provider (context,
+                               GTK_STYLE_PROVIDER (provider),
+                               GTK_STYLE_PROVIDER_PRIORITY_THEME);
+  gdk_rgba_free (bg_color);
+  g_free (color_string);
+}
+
+
+
 static gboolean
 pager_plugin_scroll_event (GtkWidget      *widget,
                            GdkEventScroll *event)
@@ -330,7 +382,7 @@ pager_plugin_screen_layout_changed (PagerPlugin *plugin)
 
   if (plugin->miniature_view)
     {
-      plugin->pager = wnck_pager_new (plugin->wnck_screen);
+      plugin->pager = wnck_pager_new ();
       wnck_pager_set_display_mode (WNCK_PAGER (plugin->pager), WNCK_PAGER_DISPLAY_CONTENT);
       if (!wnck_pager_set_n_rows (WNCK_PAGER (plugin->pager), plugin->rows))
         g_message ("Setting the pager rows returned false. Maybe the setting is not applied.");
@@ -393,14 +445,18 @@ pager_plugin_construct (XfcePanelPlugin *panel_plugin)
 
   xfce_panel_plugin_menu_show_configure (panel_plugin);
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   mi = gtk_image_menu_item_new_with_mnemonic (_("Workspace _Settings..."));
+G_GNUC_END_IGNORE_DEPRECATIONS
   xfce_panel_plugin_menu_insert_item (panel_plugin, GTK_MENU_ITEM (mi));
   g_signal_connect (G_OBJECT (mi), "activate",
       G_CALLBACK (pager_plugin_configure_workspace_settings), NULL);
   gtk_widget_show (mi);
 
   image = gtk_image_new_from_icon_name ("xfce4-workspaces", GTK_ICON_SIZE_MENU);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mi), image);
+G_GNUC_END_IGNORE_DEPRECATIONS
   gtk_widget_show (image);
 
   panel_properties_bind (NULL, G_OBJECT (plugin),
@@ -410,6 +466,8 @@ pager_plugin_construct (XfcePanelPlugin *panel_plugin)
   g_signal_connect (G_OBJECT (plugin), "screen-changed",
       G_CALLBACK (pager_plugin_screen_changed), NULL);
   pager_plugin_screen_changed (GTK_WIDGET (plugin), NULL);
+  g_signal_connect (G_OBJECT (plugin->pager), "style-updated",
+      G_CALLBACK (pager_plugin_style_updated), NULL);
 }
 
 
@@ -560,18 +618,21 @@ pager_plugin_configure_plugin (XfcePanelPlugin *panel_plugin)
 
   object = gtk_builder_get_object (builder, "workspace-scrolling");
   panel_return_if_fail (GTK_IS_TOGGLE_BUTTON (object));
-  exo_mutual_binding_new (G_OBJECT (plugin), "workspace-scrolling",
-                          G_OBJECT (object), "active");
+  g_object_bind_property (G_OBJECT (plugin), "workspace-scrolling",
+                          G_OBJECT (object), "active",
+                          G_BINDING_SYNC_CREATE | G_BINDING_BIDIRECTIONAL);
 
   object = gtk_builder_get_object (builder, "miniature-view");
   panel_return_if_fail (GTK_IS_TOGGLE_BUTTON (object));
-  exo_mutual_binding_new (G_OBJECT (plugin), "miniature-view",
-                          G_OBJECT (object), "active");
+  g_object_bind_property (G_OBJECT (plugin), "miniature-view",
+                          G_OBJECT (object), "active",
+                          G_BINDING_SYNC_CREATE | G_BINDING_BIDIRECTIONAL);
 
   object = gtk_builder_get_object (builder, "rows");
   panel_return_if_fail (GTK_IS_ADJUSTMENT (object));
-  exo_mutual_binding_new (G_OBJECT (plugin), "rows",
-                          G_OBJECT (object), "value");
+  g_object_bind_property (G_OBJECT (plugin), "rows",
+                          G_OBJECT (object), "value",
+                          G_BINDING_SYNC_CREATE | G_BINDING_BIDIRECTIONAL);
 
   /* update the rows limit */
   pager_plugin_configure_n_workspaces_changed (plugin->wnck_screen, NULL, builder);
@@ -580,44 +641,86 @@ pager_plugin_configure_plugin (XfcePanelPlugin *panel_plugin)
 }
 
 
-
 static void
-pager_plugin_size_request (GtkWidget      *widget,
-                           GtkRequisition *requisition)
+pager_plugin_get_preferred_width (GtkWidget *widget,
+                                  gint      *minimum_width,
+                                  gint      *natural_width)
 {
   PagerPlugin         *plugin = XFCE_PAGER_PLUGIN (widget);
   XfcePanelPluginMode  mode;
   gint                 n_workspaces, n_cols;
+  gint                 min_width = 0;
+  gint                 nat_width = 0;
 
-  if (plugin->miniature_view)
+  if (plugin->pager != NULL)
+    gtk_widget_get_preferred_width (plugin->pager, &min_width, &nat_width);
+
+  mode = xfce_panel_plugin_get_mode (XFCE_PANEL_PLUGIN (plugin));
+  if (mode == XFCE_PANEL_PLUGIN_MODE_VERTICAL ||
+      mode == XFCE_PANEL_PLUGIN_MODE_DESKBAR)
+    min_width = nat_width = xfce_panel_plugin_get_size (XFCE_PANEL_PLUGIN (plugin));
+  else if (plugin->miniature_view)
     {
-      mode   = xfce_panel_plugin_get_mode (XFCE_PANEL_PLUGIN (plugin));
       n_workspaces = wnck_screen_get_workspace_count (plugin->wnck_screen);
       n_cols = MAX (1, (n_workspaces + plugin->rows - 1) / plugin->rows);
-      if (mode == XFCE_PANEL_PLUGIN_MODE_HORIZONTAL)
-        {
-          requisition->height = xfce_panel_plugin_get_size (XFCE_PANEL_PLUGIN (plugin));
-          requisition->width = (gint) (requisition->height / plugin->rows * plugin->ratio * n_cols);
-        }
-      else if (mode == XFCE_PANEL_PLUGIN_MODE_VERTICAL)
-        {
-          requisition->width = xfce_panel_plugin_get_size (XFCE_PANEL_PLUGIN (plugin));
-          requisition->height = (gint) (requisition->width / plugin->rows / plugin->ratio * n_cols);
-        }
-      else /* (mode == XFCE_PANEL_PLUGIN_MODE_DESKBAR) */
-        {
-          requisition->width = xfce_panel_plugin_get_size (XFCE_PANEL_PLUGIN (plugin));
-          requisition->height = (gint) (requisition->width / n_cols / plugin->ratio * plugin->rows);
-        }
-    }
-  else if (plugin->pager)
-    {
-      gtk_widget_size_request (plugin->pager, requisition);
-    }
-  else // initial fallback
-    {
-      requisition->width = 1;
-      requisition->height = 1;
+      min_width = nat_width = (gint) (xfce_panel_plugin_get_size (XFCE_PANEL_PLUGIN (plugin)) / plugin->rows * plugin->ratio * n_cols);
     }
+
+  if (minimum_width != NULL)
+    *minimum_width = min_width;
+
+  if (natural_width != NULL)
+    *natural_width = nat_width;
 }
 
+static void
+pager_plugin_get_preferred_height (GtkWidget *widget,
+                                   gint      *minimum_height,
+                                   gint      *natural_height)
+{
+  PagerPlugin         *plugin = XFCE_PAGER_PLUGIN (widget);
+  XfcePanelPluginMode  mode;
+  gint                 n_workspaces, n_cols;
+  gint                 min_height = 0;
+  gint                 nat_height = 0;
+
+  if (plugin->pager != NULL)
+    gtk_widget_get_preferred_height (plugin->pager, &min_height, &nat_height);
+
+  mode = xfce_panel_plugin_get_mode (XFCE_PANEL_PLUGIN (plugin));
+  if (mode == XFCE_PANEL_PLUGIN_MODE_HORIZONTAL)
+    min_height = nat_height = xfce_panel_plugin_get_size (XFCE_PANEL_PLUGIN (plugin));
+  else if (plugin->miniature_view)
+    {
+      n_workspaces = wnck_screen_get_workspace_count (plugin->wnck_screen);
+      n_cols = MAX (1, (n_workspaces + plugin->rows - 1) / plugin->rows);
+      if (mode == XFCE_PANEL_PLUGIN_MODE_VERTICAL)
+        min_height = nat_height = (gint) (xfce_panel_plugin_get_size (XFCE_PANEL_PLUGIN (plugin)) / plugin->rows / plugin->ratio * n_cols);
+      else /* (mode == XFCE_PANEL_PLUGIN_MODE_DESKBAR) */
+        min_height = nat_height = (gint) (xfce_panel_plugin_get_size (XFCE_PANEL_PLUGIN (plugin)) / n_cols / plugin->ratio * plugin->rows);
+    }
+
+  if (minimum_height != NULL)
+    *minimum_height = min_height;
+
+  if (natural_height != NULL)
+    *natural_height = nat_height;
+}
+
+static void
+pager_plugin_get_preferred_width_for_height (GtkWidget *widget,
+                                             gint       height,
+                                             gint      *minimum_width,
+                                             gint      *natural_width)
+{
+  pager_plugin_get_preferred_width (widget, minimum_width, natural_width);
+}
+
+static void
+pager_plugin_get_preferred_height_for_width (GtkWidget *widget,
+                                             gint       width,
+                                             gint      *minimum_height,
+                                             gint      *natural_height)
+{
+  pager_plugin_get_preferred_height (widget, minimum_height, natural_height);
+}
