@@ -108,8 +108,13 @@ struct _SystrayBox
   /* maximum icon size */
   gint          size_max;
 
+  /* whether icons are squared */
+  guint         square_icons : 1;
+
   /* allocated size by the plugin */
+  gint          size_alloc_init;
   gint          size_alloc;
+  gint          nrows;
 };
 
 
@@ -158,11 +163,13 @@ systray_box_init (SystrayBox *box)
   box->children = NULL;
   box->names_ordered = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
   box->size_max = SIZE_MAX_DEFAULT;
+  box->size_alloc_init = SIZE_MAX_DEFAULT;
   box->size_alloc = SIZE_MAX_DEFAULT;
   box->n_hidden_children = 0;
   box->n_visible_children = 0;
   box->horizontal = TRUE;
   box->show_hidden = FALSE;
+  box->square_icons = FALSE;
 }
 
 
@@ -222,41 +229,56 @@ systray_box_size_get_max_child_size (SystrayBox *box,
   gint              row_size;
   GtkStyleContext  *ctx;
   GtkBorder         padding;
+  gint              spacing;
 
   ctx = gtk_widget_get_style_context (widget);
   gtk_style_context_get_padding (ctx, gtk_widget_get_state_flags (widget), &padding);
   alloc_size -= MAX (padding.left+padding.right, padding.top+padding.bottom);
 
-  /* count the number of rows that fit in the allocated space */
-  for (rows = 1;; rows++)
+  if (box->square_icons)
     {
-      size = rows * box->size_max + (rows - 1) * SPACING;
-      if (size < alloc_size)
-        continue;
+      if (rows_ret != NULL)
+        *rows_ret = box->nrows;
 
-      /* decrease rows if the new size doesn't fit */
-      if (rows > 1 && size > alloc_size)
-        rows--;
+      if (row_size_ret != NULL)
+        *row_size_ret = alloc_size / box->nrows;
 
-      break;
-    }
-
-  row_size = (alloc_size - (rows - 1) * SPACING) / rows;
-  row_size = MIN (box->size_max, row_size);
-
-  if (rows_ret != NULL)
-    *rows_ret = rows;
-
-  if (row_size_ret != NULL)
-    *row_size_ret = row_size;
-
-  if (offset_ret != NULL)
-    {
-      rows = MIN (rows, box->n_visible_children);
-      *offset_ret = (alloc_size - (rows * row_size + (rows - 1) * SPACING)) / 2;
-      if (*offset_ret < 1)
+      if (offset_ret != NULL)
         *offset_ret = 0;
     }
+  else
+    {
+      /* count the number of rows that fit in the allocated space */
+      for (rows = 1;; rows++)
+        {
+          size = rows * box->size_max + (rows - 1) * SPACING;
+          if (size < alloc_size)
+            continue;
+
+          /* decrease rows if the new size doesn't fit */
+          if (rows > 1 && size > alloc_size)
+            rows--;
+
+          break;
+        }
+
+      row_size = (alloc_size - (rows - 1) * SPACING) / rows;
+      row_size = MIN (box->size_max, row_size);
+
+      if (rows_ret != NULL)
+        *rows_ret = rows;
+
+      if (row_size_ret != NULL)
+        *row_size_ret = row_size;
+
+      if (offset_ret != NULL)
+        {
+          rows = MIN (rows, box->n_visible_children);
+          *offset_ret = (alloc_size - (rows * row_size + (rows - 1) * SPACING)) / 2;
+          if (*offset_ret < 1)
+            *offset_ret = 0;
+        }
+    }
 }
 
 
@@ -266,7 +288,7 @@ systray_box_get_preferred_width   (GtkWidget       *widget,
                                    gint            *minimum_width,
                                    gint            *natural_width)
 {
-  SystrayBox     *box = XFCE_SYSTRAY_BOX (widget);
+  SystrayBox *box = XFCE_SYSTRAY_BOX (widget);
 
   if (box->horizontal)
     {
@@ -275,26 +297,27 @@ systray_box_get_preferred_width   (GtkWidget       *widget,
   else
     {
       if (minimum_width != NULL)
-        *minimum_width = -1;
+        *minimum_width = box->size_alloc_init;
       if (natural_width != NULL)
-        *natural_width = -1;
+        *natural_width = box->size_alloc_init;
     }
 }
 
 
+
 static void
 systray_box_get_preferred_height  (GtkWidget       *widget,
                                    gint            *minimum_height,
                                    gint            *natural_height)
 {
-  SystrayBox     *box = XFCE_SYSTRAY_BOX (widget);
+  SystrayBox *box = XFCE_SYSTRAY_BOX (widget);
 
   if (box->horizontal)
     {
       if (minimum_height != NULL)
-        *minimum_height = -1;
+        *minimum_height = box->size_alloc_init;
       if (natural_height != NULL)
-        *natural_height = -1;
+        *natural_height = box->size_alloc_init;
     }
   else
     {
@@ -303,6 +326,7 @@ systray_box_get_preferred_height  (GtkWidget       *widget,
 }
 
 
+
 static void
 systray_box_get_preferred_length (GtkWidget      *widget,
                                   gint           *minimum_length,
@@ -351,7 +375,7 @@ systray_box_get_preferred_length (GtkWidget      *widget,
           /* special handling for non-squared icons. this only works if
            * the icon size ratio is > 1.00, if this is lower then 1.00
            * the icon implementation should respect the tray orientation */
-          if (G_UNLIKELY (child_req.width != child_req.height))
+          if (!box->square_icons && G_UNLIKELY (child_req.width != child_req.height))
             {
               ratio = (gdouble) child_req.width / (gdouble) child_req.height;
               if (!box->horizontal)
@@ -398,7 +422,10 @@ systray_box_get_preferred_length (GtkWidget      *widget,
       if (min_seq_cells != -1)
         cols = MAX (min_seq_cells, cols);
 
-      length = row_size * cols + (cols - 1) * SPACING;
+      if (box->square_icons)
+        length = row_size * cols;
+      else
+        length = row_size * cols + (cols - 1) * SPACING;
     }
   else
     {
@@ -449,6 +476,7 @@ systray_box_size_allocate (GtkWidget     *widget,
   gint              idx;
   GtkStyleContext  *ctx;
   GtkBorder         padding;
+  gint              spacing;
 
   gtk_widget_set_allocation (widget, allocation);
 
@@ -456,6 +484,7 @@ systray_box_size_allocate (GtkWidget     *widget,
   gtk_style_context_get_padding (ctx, gtk_widget_get_state_flags (widget), &padding);
 
   alloc_size = box->horizontal ? allocation->height : allocation->width;
+  spacing = box->square_icons ? 0 : SPACING;
 
   systray_box_size_get_max_child_size (box, alloc_size, &rows, &row_size, &offset);
 
@@ -507,7 +536,7 @@ systray_box_size_allocate (GtkWidget     *widget,
       else
         {
           /* special case handling for non-squared icons */
-          if (G_UNLIKELY (child_req.width != child_req.height))
+          if (!box->square_icons && G_UNLIKELY (child_req.width != child_req.height))
             {
               ratio = (gdouble) child_req.width / (gdouble) child_req.height;
 
@@ -541,10 +570,20 @@ systray_box_size_allocate (GtkWidget     *widget,
           else
             {
               /* fix icon to row size */
-              child_alloc.width = row_size;
-              child_alloc.height = row_size;
-              child_alloc.x = 0;
-              child_alloc.y = 0;
+              if (box->square_icons)
+                {
+                  child_alloc.width = MIN (row_size, box->size_max);
+                  child_alloc.height = MIN (row_size, box->size_max);
+                  child_alloc.x = (row_size - child_alloc.width) / 2;
+                  child_alloc.y = (row_size - child_alloc.height) / 2;
+                }
+              else
+                {
+                  child_alloc.width = row_size;
+                  child_alloc.height = row_size;
+                  child_alloc.x = 0;
+                  child_alloc.y = 0;
+                }
 
               ratio = 1.00;
             }
@@ -568,9 +607,9 @@ systray_box_size_allocate (GtkWidget     *widget,
               if (box->horizontal)
                 {
                   x = x_start;
-                  y += row_size + SPACING;
+                  y += row_size + spacing;
 
-                  if (y > y_end)
+                  if (!box->square_icons && y > y_end)
                     {
                       /* we overflow the number of rows, restart
                        * allocation with 1px smaller icons */
@@ -586,9 +625,9 @@ systray_box_size_allocate (GtkWidget     *widget,
               else
                 {
                   y = y_start;
-                  x += row_size + SPACING;
+                  x += row_size + spacing;
 
-                  if (x > x_end)
+                  if (!box->square_icons && x > x_end)
                     {
                       /* we overflow the number of rows, restart
                        * allocation with 1px smaller icons */
@@ -607,9 +646,9 @@ systray_box_size_allocate (GtkWidget     *widget,
           child_alloc.y += y;
 
           if (box->horizontal)
-            x += row_size * ratio + SPACING;
+            x += row_size * ratio + spacing;
           else
-            y += row_size * ratio + SPACING;
+            y += row_size * ratio + spacing;
         }
 
       panel_debug_filtered (PANEL_DEBUG_SYSTRAY, "allocated %s[%p] at (%d,%d;%d,%d)",
@@ -618,6 +657,13 @@ systray_box_size_allocate (GtkWidget     *widget,
 
       gtk_widget_size_allocate (child, &child_alloc);
     }
+
+  /* recalculate size with higher precise */
+  if (alloc_size != box->size_alloc)
+    {
+      box->size_alloc = alloc_size;
+      gtk_widget_queue_resize (GTK_WIDGET (box));
+    }
 }
 
 
@@ -715,9 +761,9 @@ systray_box_compare_function (gconstpointer a,
   name_a = systray_socket_get_name (XFCE_SYSTRAY_SOCKET (a));
   name_b = systray_socket_get_name (XFCE_SYSTRAY_SOCKET (b));
 
-  if (g_hash_table_lookup_extended (box->names_ordered, name_a, NULL, &value))
+  if (name_a != NULL && g_hash_table_lookup_extended (box->names_ordered, name_a, NULL, &value))
     index_a = GPOINTER_TO_INT (value);
-  if (g_hash_table_lookup_extended (box->names_ordered, name_b, NULL, &value))
+  if (name_b != NULL && g_hash_table_lookup_extended (box->names_ordered, name_b, NULL, &value))
     index_b = GPOINTER_TO_INT (value);
 
   /* sort ordered icons before unordered ones */
@@ -802,13 +848,16 @@ systray_box_get_size_max (SystrayBox *box)
 
 void
 systray_box_set_size_alloc (SystrayBox *box,
-                            gint        size_alloc)
+                            gint        size_alloc,
+                            gint        nrows)
 {
   panel_return_if_fail (XFCE_IS_SYSTRAY_BOX (box));
 
-  if (G_LIKELY (size_alloc != box->size_alloc))
+  if (G_LIKELY (size_alloc != box->size_alloc || nrows != box->nrows))
     {
+      box->size_alloc_init = size_alloc;
       box->size_alloc = size_alloc;
+      box->nrows = nrows;
 
       if (box->children != NULL)
         gtk_widget_queue_resize (GTK_WIDGET (box));
@@ -819,7 +868,7 @@ systray_box_set_size_alloc (SystrayBox *box,
 
 void
 systray_box_set_show_hidden (SystrayBox *box,
-                              gboolean   show_hidden)
+                             gboolean    show_hidden)
 {
   panel_return_if_fail (XFCE_IS_SYSTRAY_BOX (box));
 
@@ -844,6 +893,33 @@ systray_box_get_show_hidden (SystrayBox *box)
 
 
 
+void
+systray_box_set_squared (SystrayBox *box,
+                         gboolean    square_icons)
+{
+  panel_return_if_fail (XFCE_IS_SYSTRAY_BOX (box));
+
+  if (box->square_icons != square_icons)
+    {
+      box->square_icons = square_icons;
+
+      if (box->children != NULL)
+        gtk_widget_queue_resize (GTK_WIDGET (box));
+    }
+}
+
+
+
+gboolean
+systray_box_get_squared (SystrayBox *box)
+{
+  panel_return_val_if_fail (XFCE_IS_SYSTRAY_BOX (box), FALSE);
+
+  return box->square_icons;
+}
+
+
+
 void
 systray_box_update (SystrayBox *box,
                     GSList     *names_ordered)
